"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var emscripten_wasm_loader_1 = require("emscripten-wasm-loader");
var nanoid = require("nanoid");
var mountBuffer_1 = require("./mountBuffer");
var mountDirectory_1 = require("./mountDirectory");
var unmount_1 = require("./unmount");
var logger_1 = require("./util/logger");
var wrapHunspellInterface_1 = require("./wrapHunspellInterface");
/**
 * Creates a factory function for mounting files into wasm filesystem
 * and creating hunspell instance.
 *
 * @param {HunspellAsmModule} asmModule wasm / asm module loaded into memory.
 *
 * @return {HunspellFactory} factory function for mounting files and creating hunspell instance.
 */
/** @internal */
exports.hunspellLoader = function (asmModule, environment) {
    var cwrap = asmModule.cwrap, FS = asmModule.FS, _free = asmModule._free, allocateUTF8 = asmModule.allocateUTF8, _malloc = asmModule._malloc, getValue = asmModule.getValue, Pointer_stringify = asmModule.Pointer_stringify;
    var hunspellInterface = wrapHunspellInterface_1.wrapHunspellInterface(cwrap);
    //creating top-level path to mount files
    var memPathId = "/" + nanoid(45);
    FS.mkdir(memPathId);
    logger_1.log("hunspellLoader: mount path for bufferFile created at " + memPathId);
    var nodePathId = "/" + nanoid(45);
    if (environment === emscripten_wasm_loader_1.ENVIRONMENT.NODE) {
        FS.mkdir(nodePathId);
        logger_1.log("hunspellLoader: mount path for directory created at " + nodePathId);
    }
    /**
     * Naive auto-dispose interface to call hunspell interface with string params.
     *
     */
    var usingParamPtr = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var params = args.slice();
        var fn = params.pop();
        var paramsPtr = params.map(function (param) { return allocateUTF8(param); });
        var ret = fn.apply(void 0, paramsPtr);
        paramsPtr.forEach(function (paramPtr) { return _free(paramPtr); });
        return ret;
    };
    return {
        mountDirectory: mountDirectory_1.mountDirectory(FS, nodePathId, environment),
        mountBuffer: mountBuffer_1.mountBuffer(FS, memPathId),
        unmount: unmount_1.unmount(FS, memPathId),
        create: function (affPath, dictPath) {
            var affPathPtr = allocateUTF8(affPath);
            var dictPathPtr = allocateUTF8(dictPath);
            var hunspellPtr = hunspellInterface.create(affPathPtr, dictPathPtr);
            return {
                dispose: function () {
                    hunspellInterface.destroy(hunspellPtr);
                    _free(affPathPtr);
                    _free(dictPathPtr);
                },
                spell: function (word) { return !!usingParamPtr(word, function (wordPtr) { return hunspellInterface.spell(hunspellPtr, wordPtr); }); },
                suggest: function (word) {
                    var suggestionListPtr = _malloc(4);
                    var suggestionCount = usingParamPtr(word, function (wordPtr) {
                        return hunspellInterface.suggest(hunspellPtr, suggestionListPtr, wordPtr);
                    });
                    var suggestionListValuePtr = getValue(suggestionListPtr, '*');
                    var ret = suggestionCount > 0
                        ? Array.from(Array(suggestionCount).keys()).map(function (idx) {
                            return Pointer_stringify(getValue(suggestionListValuePtr + idx * 4, '*'));
                        })
                        : [];
                    hunspellInterface.free_list(hunspellPtr, suggestionListPtr, suggestionCount);
                    _free(suggestionListPtr);
                    return ret;
                },
                addDictionary: function (dictPath) {
                    return usingParamPtr(dictPath, function (dictPathPtr) { return hunspellInterface.add_dic(hunspellPtr, dictPathPtr); }) === 1
                        ? false
                        : true;
                },
                addWord: function (word) { return usingParamPtr(word, function (wordPtr) { return hunspellInterface.add(hunspellPtr, wordPtr); }); },
                addWordWithAffix: function (word, affix) {
                    return usingParamPtr(word, affix, function (wordPtr, affixPtr) {
                        return hunspellInterface.add_with_affix(hunspellPtr, wordPtr, affixPtr);
                    });
                },
                removeWord: function (word) { return usingParamPtr(word, function (wordPtr) { return hunspellInterface.remove(hunspellPtr, wordPtr); }); }
            };
        }
    };
};
//# sourceMappingURL=hunspellLoader.js.map